{% load form_helpers %}
{% load i18n %}
<form action="{{ form_url }}" method="post">
  {% csrf_token %}

  <div class="modal-header">
    <h5 class="modal-title">{% trans "Confirm Merge" %}</h5>
  </div>
  <div class="modal-body">
    {% blocktrans %}
    <p>Are you sure you want to <strong class="text-danger">merge</strong> {{ object_type }} <strong>{{ object }}</strong>?</p>
    {% endblocktrans %}
    {% render_form form %}
  </div>
  <div class="modal-footer">
    {% if return_url %}
      <a href="{{ return_url }}" class="btn btn-outline-secondary">{% trans "Cancel" %}</a>
    {% else %}
      <button type="button" class="btn btn-outline-secondary" data-bs-dismiss="modal">{% trans "Cancel" %}</button>
    {% endif %}
    <button type="submit" class="btn btn-danger">{% trans "Merge" %}</button>
  </div>
  </div>
</form>
